#!/bin/env perl.exe

use strict;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::TLE::API::Template;
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::WebInterface::API::Config qw (GetRequestRouterMonitors);
use DE_EPAGES::WebInterface::API::RequestRouterMonitor;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::WebInterface::API::ServerConfig qw ( ServerConfig );
use DE_EPAGES::WebInterface::API::ASPoolDBCacheServer;

my $BaseDir = $0;
$BaseDir =~ s/[\\\/][^\\\/]+$//x;
my $TemplateFile = $BaseDir . '/server.tmpl';
my %TEMPLATE_REQUEST = map { ($_ => undef) } qw( server debug rr pool asc );

#========================================================================================
# function     Parse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = Parse();
#----------------------------------------------------------------------------------------
# description  Parses web form data of of the CGI script.
#               Joins multiple value of the same name with \1 characters.
# return       $hValues | form values | ref.hash.string
#========================================================================================
sub Parse {
    # Get the input
    my $buffer = $ENV{QUERY_STRING};
    my %FORM;
    return \%FORM unless defined $buffer;

    if( $ENV{'REQUEST_METHOD'} eq 'POST' ) {
        my $postData;
        read STDIN, $postData, $ENV{'CONTENT_LENGTH'};
        $buffer = $postData.'&'.$buffer;
    }

    # Split the name-value pairs
    my @pairs = split(/&/, $buffer);
    foreach my $pair (@pairs) {
        my($name, $value) = split(/=/, $pair);
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/egx;
        if( exists $FORM{$name} ) {
            $FORM{$name} .= "\x{1}$value";
        }
        else {
            $FORM{$name} = $value;
        }
    }

    return \%FORM;
}

sub _CacheServer {
    my ($hCacheInfo, $Host, $Port) = @_;

    my $hServerInfo = $hCacheInfo->{$Host.':'.$Port};
    return {
        'ServerIP' => $Host,
        'ServerPort' => $Port,
        $hServerInfo
        ? (
            'Online' => 1,
            'GoodHits' => $hServerInfo->{'good'}->{'hits'},
            'GoodMisses' => $hServerInfo->{'good'}->{'misses'},
            'GoodSize' => $hServerInfo->{'good'}->{'size'},
            'GoodGC' => $hServerInfo->{'good'}->{'gc'},
            'BadHits' => $hServerInfo->{'bad'}->{'hits'},
            'BadMisses' => $hServerInfo->{'bad'}->{'misses'},
            'BadSize' => $hServerInfo->{'bad'}->{'size'},
            'BadGC' => $hServerInfo->{'bad'}->{'gc'},
        )
        : (
            'Online' => 0,
        )
    }
}


sub CacheServers {
    my ($CacheServer) = @_;

    my %CacheInfo = map { $_->{'host'}.':'.$_->{'port'} => $_ }
        @{ $CacheServer->requestCacheInfo };

    return [
        map { _CacheServer(\%CacheInfo, $_->{'Host'}, $_->{'Port'}) }
        @{ ServerConfig()->asPoolDBCacheServers }
    ];
}

#--------------------------------------------------------------------------
# function main
#--------------------------------------------------------------------------

sub SortLoop {
    my ($aLoop, $OrderBy, $Numeric, $Desc) = @_;
    return [
        sort {
            $Numeric
            ?
                $Desc
                ? $b->{$OrderBy} <=> $a->{$OrderBy}
                : $a->{$OrderBy} <=> $b->{$OrderBy}
            :
                $Desc
                ? $b->{$OrderBy} cmp $a->{$OrderBy}
                : $a->{$OrderBy} cmp $b->{$OrderBy}
        }
        @$aLoop
    ]
}

sub main {

    my $Form = Parse();

    my @Servers = defined $Form->{'Server'} ? split /\x{1}/, $Form->{'Server'} : ();
    if (not scalar @Servers) {
        my $aMCs = GetRequestRouterMonitors();
        @Servers = map { $_->[0].':'.$_->[1] } @$aMCs;
    }

    my $Template = $Form->{'Template'};
    $Template = 'server' unless defined $Template;
    $Template = 'server' if $Template eq 'sas60';
    $Template = 'debug'  if $Template eq 'sas';
    $Template = 'server' unless exists $TEMPLATE_REQUEST{$Template};

    my $OrderBy = $Form->{OrderBy};
    my $Numeric = $Form->{Numeric} || 0;
    my $Desc = $Form->{Desc} || 0;

    my @ServerLoop;
    if( $Template eq 'asc' ) {
        my $ASC = DE_EPAGES::WebInterface::API::ASPoolDBCacheServer->new;
        $ASC->resetStatistics() if($Form->{'resetStatistics'});
        $ASC->resetCache() if($Form->{'clearCaches'});
        @ServerLoop = @{SortLoop( CacheServers($ASC), $OrderBy, $Numeric, $Desc )};
    }
    else {
        foreach my $Server ( @Servers ) {
            my ($ServerName, $MonitorPort) = $Server =~ /(.+)\:(.+)/;
            die "Invalid server format: $Server" unless defined $ServerName;

            my $hVars = {};
            eval {
                my $Monitor = DE_EPAGES::WebInterface::API::RequestRouterMonitor->new(
                    Address => $ServerName,
                    Port => $MonitorPort
                );

                if($Form->{'resetStatisticsForAllPools'}) {
                    $Monitor->resetStatisticsForAllPools;
                }
                elsif($Form->{'resetStatisticsForPool'}) {
                    my $PoolName = $Form->{'resetStatisticsForPool'};
                    $Monitor->resetStatisticsForPool($PoolName);
                }

                if( $Template eq 'server' ) {
                    $hVars->{'ApplicationServers'} = SortLoop( $Monitor->servers,
                        $OrderBy, $Numeric, $Desc );
                }
                elsif( $Template eq 'debug' ) {
                    $hVars->{'ApplicationServers'} = SortLoop( $Monitor->debug,
                        $OrderBy, $Numeric, $Desc );
                }
                elsif( $Template eq 'rr' ) {
                    $hVars->{'RequestRouters'} = SortLoop( $Monitor->requestRouters,
                        $OrderBy, $Numeric, $Desc );
                }
                elsif( $Template eq 'pool' ) {
                    my $Pools = $Monitor->pools;
                    $_->{'RRPreference'} = join(', ', @{$_->{'RRPreference'}}) foreach (@$Pools);
                    $hVars->{'Pools'} = SortLoop( $Pools,
                        $OrderBy, $Numeric, $Desc );
                }

                $hVars->{'ConnectionFailed'}  = 0;
            };
            if (ExistsError()) {
                my $Error = GetError();
                $hVars->{'ConnectionFailed'}  = 1;
                $hVars->{'FailedString'}      = $Error->shortMessage;
            }
            $hVars->{'ServerName'}  = $ServerName;
            $hVars->{'MonitorPort'} = $MonitorPort;
            $hVars->{'Server'} = $ServerName.':'.$MonitorPort;
            push @ServerLoop, $hVars;
        }
    }
    my ($sec,$min,$hour,$mday,$mon,$year) = localtime( time );

    my %Vars = (
        'Template'      => $Template,
        'OrderBy'       => $OrderBy,
        'Numeric'       => $Numeric,
        'Desc'          => $Desc,
        'Servers'       => \@ServerLoop,
        'CurrentTime'   => sprintf( '%02d:%02d:%02d', $hour, $min, $sec ),
        'CurrentDate'   => sprintf( '%02d-%02d-%02d', 1900+$year, $mon+1, $mday ), ## no critic qw(ProhibitMagicNumbers)
        );

    %Vars = (%Vars, %ENV);

    if( exists $Form->{'Template'} ) {
        $TemplateFile = $BaseDir .'/'. $Form->{'Template'} .'.tmpl';
    }

    my $TLEProcessor = DE_EPAGES::TLE::API::BaseProcessor->new;
    $TLEProcessor->registerHandler('FunctionHandler', 'SpyTLE', 'ColumnLink');
    my $NoCtmplTemplate = NoCtmplTemplate->new($TLEProcessor);
    print $NoCtmplTemplate->process( $TemplateFile, \%Vars );

    return;
}

eval {

    RunScript(
        'Sub' => \&main
    );

};
if( $@ ) {
    print "Content-Type: text/html\n\n";
    print "<HTML><HEAD><TITLE>ScriptError</TITLE></HEAD>\n";
    print '<H1>Script Error</H1>';
    print "<PRE>$@</PRE>\n";
    print "</BODY></HTML>\n";
}

## no critic(ProhibitMultiplePackages)

#========================================================================================
# package      NoCtmplTemplate
# state        private
#========================================================================================
package NoCtmplTemplate;
use base 'DE_EPAGES::TLE::API::Template';

sub perlFileName {
    return undef;
}

#========================================================================================
# package      SpyTLE
# state        private
#========================================================================================
package SpyTLE;

sub ColumnLink {
    my $self = shift;
    my ($Processor, $raParams) = @_;

    my $Name = $raParams->[0];
    my @Types = split(/\s/, lc($raParams->[1] // ''));
    my $DisplayName = $raParams->[2] // $Name;

    my $OrderBy = $Processor->tle('OrderBy');
    my $Desc = ($Processor->tle('Desc') || $OrderBy ne $Name) ? 0 : 1;
    my $Arrow = ('arrow' ~~ @Types && $OrderBy eq $Name) ? '&nbsp;' . ($Desc ? '&uarr;' : '&darr;') : '';
    my $Numeric = 'numeric' ~~ @Types ? 1 : 0;
    my $Query = 'Template=' . $Processor->tle('Template');

    return "<a href='?$Query&amp;OrderBy=$Name&amp;Numeric=$Numeric&amp;Desc=$Desc'>$DisplayName</a>$Arrow";
}
